//
// Copyright © 2024 Stream.io Inc. All rights reserved.
//

import StreamVideo
import SwiftUI

public struct ReconnectionView<Factory: ViewFactory>: View {
    
    @Injected(\.colors) var colors
    
    @ObservedObject var viewModel: CallViewModel
    var viewFactory: Factory
    
    public init(
        viewModel: CallViewModel,
        viewFactory: Factory
    ) {
        self.viewModel = viewModel
        self.viewFactory = viewFactory
    }
    
    public var body: some View {
        WaitingLocalUserView(viewModel: viewModel, viewFactory: viewFactory)
            .overlay(
                VStack {
                    Text(L10n.Call.Current.reconnecting)
                        .applyCallingStyle()
                        .padding()
                        .accessibility(identifier: "reconnectingMessage")
                    CallingIndicator()
                }
                .padding()
                .background(
                    Color(colors.callBackground).opacity(0.7).edgesIgnoringSafeArea(.all)
                )
                .cornerRadius(16)
            )
    }
}
